@@ -2,9 +2,11 @@
 # =================================================================
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
+#          Ricardo Garcia Silva <ricardo.garcia.silva@gmail.com>
 #
 # Copyright (c) 2017 Tom Kralidis
 # Copyright (c) 2016 James F. Dickens
+# Copyright (c) 2017 Ricardo Garcia Silva
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -33,10 +35,14 @@ import logging
 import uuid
 from six.moves import range
 from six.moves.urllib.parse import urlparse
-from owslib.util import build_get_url
+
 from geolinks import sniff_link
-from pycsw.core import util
+from owslib.util import build_get_url
+from shapely.wkt import loads
+from shapely.geometry import MultiPolygon
+
 from pycsw.core.etree import etree
+from pycsw.core import util
 
 LOGGER = logging.getLogger(__name__)
 
@@ -51,7 +57,7 @@ def parse_record(context, record, repos=None,
     # parse web services
     if (mtype == 'http://www.opengis.net/cat/csw/2.0.2' and
         isinstance(record, str) and record.startswith('http')):
-        LOGGER.debug('CSW service detected, fetching via HTTP')
+        LOGGER.info('CSW service detected, fetching via HTTP')
         # CSW service, not csw:Record
         try:
             return _parse_csw(context, repos, record, identifier, pagesize)
@@ -59,7 +65,7 @@ def parse_record(context, record, repos=None,
             # TODO: implement better exception handling
             if str(err).find('ExceptionReport') != -1:
                 msg = 'CSW harvesting error: %s' % str(err)
-                LOGGER.debug(msg)
+                LOGGER.exception(msg)
                 raise RuntimeError(msg)
             LOGGER.debug('Not a CSW, attempting to fetch Dublin Core')
             try:
@@ -69,40 +75,44 @@ def parse_record(context, record, repos=None,
             return [_parse_dc(context, repos, etree.fromstring(content, context.parser))]
 
     elif mtype == 'urn:geoss:waf':  # WAF
-        LOGGER.debug('WAF detected, fetching via HTTP')
+        LOGGER.info('WAF detected, fetching via HTTP')
         return _parse_waf(context, repos, record, identifier)
 
     elif mtype == 'http://www.opengis.net/wms':  # WMS
-        LOGGER.debug('WMS detected, fetching via OWSLib')
+        LOGGER.info('WMS detected, fetching via OWSLib')
         return _parse_wms(context, repos, record, identifier)
 
     elif mtype == 'http://www.opengis.net/wmts/1.0':  # WMTS
-        LOGGER.debug('WMTS 1.0.0 detected, fetching via OWSLib')
+        LOGGER.info('WMTS 1.0.0 detected, fetching via OWSLib')
         return _parse_wmts(context, repos, record, identifier)
 
     elif mtype == 'http://www.opengis.net/wps/1.0.0':  # WPS
-        LOGGER.debug('WPS detected, fetching via OWSLib')
-        return [_parse_wps(context, repos, record, identifier)]
+        LOGGER.info('WPS detected, fetching via OWSLib')
+        return _parse_wps(context, repos, record, identifier)
 
-    elif mtype == 'http://www.opengis.net/wfs':  # WFS
-        LOGGER.debug('WFS detected, fetching via OWSLib')
-        return _parse_wfs(context, repos, record, identifier)
+    elif mtype == 'http://www.opengis.net/wfs':  # WFS 1.1.0
+        LOGGER.info('WFS detected, fetching via OWSLib')
+        return _parse_wfs(context, repos, record, identifier, '1.1.0')
+
+    elif mtype == 'http://www.opengis.net/wfs/2.0':  # WFS 2.0.0
+        LOGGER.info('WFS detected, fetching via OWSLib')
+        return _parse_wfs(context, repos, record, identifier, '2.0.0')
 
     elif mtype == 'http://www.opengis.net/wcs':  # WCS
-        LOGGER.debug('WCS detected, fetching via OWSLib')
+        LOGGER.info('WCS detected, fetching via OWSLib')
         return _parse_wcs(context, repos, record, identifier)
 
     elif mtype == 'http://www.opengis.net/sos/1.0':  # SOS 1.0.0
-        LOGGER.debug('SOS 1.0.0 detected, fetching via OWSLib')
+        LOGGER.info('SOS 1.0.0 detected, fetching via OWSLib')
         return _parse_sos(context, repos, record, identifier, '1.0.0')
 
     elif mtype == 'http://www.opengis.net/sos/2.0':  # SOS 2.0.0
-        LOGGER.debug('SOS 2.0.0 detected, fetching via OWSLib')
+        LOGGER.info('SOS 2.0.0 detected, fetching via OWSLib')
         return _parse_sos(context, repos, record, identifier, '2.0.0')
 
     elif (mtype == 'http://www.opengis.net/cat/csw/csdgm' and
           record.startswith('http')):  # FGDC
-        LOGGER.debug('FGDC detected, fetching via HTTP')
+        LOGGER.info('FGDC detected, fetching via HTTP')
         record = util.http_request('GET', record)
 
     return _parse_metadata(context, repos, record)
@@ -124,7 +134,7 @@ def _parse_metadata(context, repos, record):
 
     root = exml.tag
 
-    LOGGER.debug('Serialized metadata, parsing content model')
+    LOGGER.info('Serialized metadata, parsing content model')
 
     if root == '{%s}MD_Metadata' % context.namespaces['gmd']:  # ISO
         return [_parse_iso(context, repos, exml)]
@@ -135,6 +145,8 @@ def _parse_metadata(context, repos, record):
         return [_parse_fgdc(context, repos, exml)]
     elif root == '{%s}TRANSFER' % context.namespaces['gm03']:  # GM03
         return [_parse_gm03(context, repos, exml)]
+    elif root == '{http://www.geocat.ch/2008/che}CHE_MD_Metadata': # GM03 ISO profile
+        return [_parse_iso(context, repos, exml)]
     elif root == '{%s}Record' % context.namespaces['csw']:  # Dublin Core CSW
         return [_parse_dc(context, repos, exml)]
     elif root == '{%s}RDF' % context.namespaces['rdf']:  # Dublin Core RDF
@@ -155,14 +167,19 @@ def _parse_csw(context, repos, record, identifier, pagesize=10):
     # if init raises error, this might not be a CSW
     md = CatalogueServiceWeb(record, timeout=60)
 
-    LOGGER.debug('Setting CSW service metadata')
+    LOGGER.info('Setting CSW service metadata')
     # generate record of service instance
     _set(context, serviceobj, 'pycsw:Identifier', identifier)
     _set(context, serviceobj, 'pycsw:Typename', 'csw:Record')
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/cat/csw/2.0.2')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(md._exml))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(md._exml)
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -200,7 +217,7 @@ def _parse_csw(context, repos, record, identifier, pagesize=10):
     grop = md.get_operation_by_name('GetRecords')
     if all(['gmd:MD_Metadata' in grop.parameters['typeNames']['values'],
             'http://www.isotc211.org/2005/gmd' in grop.parameters['outputSchema']['values']]):
-        LOGGER.info('CSW supports ISO')
+        LOGGER.debug('CSW supports ISO')
         csw_typenames = 'gmd:MD_Metadata'
         csw_outputschema = 'http://www.isotc211.org/2005/gmd'
 
@@ -218,7 +235,7 @@ def _parse_csw(context, repos, record, identifier, pagesize=10):
     if pagesize > matches:
         pagesize = matches
 
-    LOGGER.debug('Harvesting %d CSW records' % matches)
+    LOGGER.info('Harvesting %d CSW records', matches)
 
     # loop over all catalogue records incrementally
     for r in range(1, matches+1, pagesize):
@@ -230,7 +247,7 @@ def _parse_csw(context, repos, record, identifier, pagesize=10):
         for k, v in md.records.items():
             # try to parse metadata
             try:
-                LOGGER.debug('Parsing metadata record: %s', v.xml)
+                LOGGER.info('Parsing metadata record: %s', v.xml)
                 if csw_typenames == 'gmd:MD_Metadata':
                     recobjs.append(_parse_iso(context, repos,
                                               etree.fromstring(v.xml, context.parser)))
@@ -238,7 +255,7 @@ def _parse_csw(context, repos, record, identifier, pagesize=10):
                     recobjs.append(_parse_dc(context, repos,
                                              etree.fromstring(v.xml, context.parser)))
             except Exception as err:  # parsing failed for some reason
-                LOGGER.warning('Metadata parsing failed %s', err)
+                LOGGER.exception('Metadata parsing failed')
 
     return recobjs
 
@@ -259,7 +276,7 @@ def _parse_waf(context, repos, record, identifier):
     up = urlparse(record)
     links = []
 
-    LOGGER.debug('collecting links')
+    LOGGER.info('collecting links')
     for link in tree.xpath('//a/@href'):
         link = link.strip()
         if not link:
@@ -282,7 +299,7 @@ def _parse_waf(context, repos, record, identifier):
 
     LOGGER.debug('%d links found', len(links))
     for link in links:
-        LOGGER.debug('Processing link %s', link)
+        LOGGER.info('Processing link %s', link)
         # fetch and parse
         linkcontent = util.http_request('GET', link)
         recobj = _parse_metadata(context, repos, linkcontent)[0]
@@ -300,6 +317,11 @@ def _parse_wms(context, repos, record, identifier):
     serviceobj = repos.dataset()
 
     md = WebMapService(record)
+    try:
+        md = WebMapService(record, version='1.3.0')
+    except Exception as err:
+        LOGGER.info('Looks like WMS 1.3.0 is not supported; trying 1.1.1', err)
+        md = WebMapService(record)
 
     # generate record of service instance
     _set(context, serviceobj, 'pycsw:Identifier', identifier)
@@ -307,7 +329,12 @@ def _parse_wms(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/wms')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(md.getServiceXML()))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(md.getServiceXML())
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -345,7 +372,7 @@ def _parse_wms(context, repos, record, identifier):
 
     # generate record foreach layer
 
-    LOGGER.debug('Harvesting %d WMS layers' % len(md.contents))
+    LOGGER.info('Harvesting %d WMS layers', len(md.contents))
 
     for layer in md.contents:
         recobj = repos.dataset()
@@ -361,10 +388,16 @@ def _parse_wms(context, repos, record, identifier):
         _set(context, recobj, 'pycsw:Abstract', md.contents[layer].abstract)
         _set(context, recobj, 'pycsw:Keywords', ','.join(md.contents[layer].keywords))
 
-        _set(context, recobj, 'pycsw:AnyText',
-             util.get_anytext([md.contents[layer].title,
-                              md.contents[layer].abstract,
-                              ','.join(md.contents[layer].keywords)]))
+        _set(
+            context,
+            recobj,
+            'pycsw:AnyText',
+            util.get_anytext([
+                md.contents[layer].title,
+                md.contents[layer].abstract,
+                ','.join(md.contents[layer].keywords)
+            ])
+        )
 
         bbox = md.contents[layer].boundingBoxWGS84
         if bbox is not None:
@@ -433,7 +466,12 @@ def _parse_wmts(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/wmts/1.0')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(md.getServiceXML()))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(md.getServiceXML())
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -472,7 +510,7 @@ def _parse_wmts(context, repos, record, identifier):
 
     # generate record for each layer
 
-    LOGGER.debug('Harvesting %d WMTS layers' % len(md.contents))
+    LOGGER.debug('Harvesting %d WMTS layers', len(md.contents))
 
     for layer in md.contents:
         recobj = repos.dataset()
@@ -497,11 +535,16 @@ def _parse_wmts(context, repos, record, identifier):
             keywords = ','.join(md.contents[layer].keywords)
         _set(context, recobj, 'pycsw:Keywords', keywords)
 
-        _set(context, recobj, 'pycsw:AnyText',
-             util.get_anytext([md.contents[layer].title,
-                              md.contents[layer].abstract,
-                             ','.join(keywords)
-                             ]))
+        _set(
+            context,
+            recobj,
+            'pycsw:AnyText',
+             util.get_anytext([
+                 md.contents[layer].title,
+                 md.contents[layer].abstract,
+                 ','.join(keywords)
+             ])
+        )
 
         bbox = md.contents[layer].boundingBoxWGS84
 
@@ -539,7 +582,7 @@ def _parse_wmts(context, repos, record, identifier):
     return recobjs
 
 
-def _parse_wfs(context, repos, record, identifier):
+def _parse_wfs(context, repos, record, identifier, version):
 
     from owslib.wfs import WebFeatureService
 
@@ -547,7 +590,11 @@ def _parse_wfs(context, repos, record, identifier):
     recobjs = []
     serviceobj = repos.dataset()
 
-    md = WebFeatureService(record, '1.1.0')
+    try:
+        md = WebFeatureService(record, version)
+    except Exception as err:
+        if version == '1.1.0':
+            md = WebFeatureService(record, '1.0.0')
 
     # generate record of service instance
     _set(context, serviceobj, 'pycsw:Identifier', identifier)
@@ -555,7 +602,12 @@ def _parse_wfs(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/wfs')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(etree.tostring(md._capabilities)))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(etree.tostring(md._capabilities))
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -584,7 +636,7 @@ def _parse_wfs(context, repos, record, identifier):
 
     # generate record foreach featuretype
 
-    LOGGER.debug('Harvesting %d WFS featuretypes' % len(md.contents))
+    LOGGER.info('Harvesting %d WFS featuretypes', len(md.contents))
 
     for featuretype in md.contents:
         recobj = repos.dataset()
@@ -600,10 +652,16 @@ def _parse_wfs(context, repos, record, identifier):
         _set(context, recobj, 'pycsw:Abstract', md.contents[featuretype].abstract)
         _set(context, recobj, 'pycsw:Keywords', ','.join(md.contents[featuretype].keywords))
 
-        _set(context, recobj, 'pycsw:AnyText',
-             util.get_anytext([md.contents[featuretype].title,
-                              md.contents[featuretype].abstract,
-                              ','.join(md.contents[featuretype].keywords)]))
+        _set(
+            context,
+            recobj,
+            'pycsw:AnyText',
+            util.get_anytext([
+                md.contents[featuretype].title,
+                md.contents[featuretype].abstract,
+                ','.join(md.contents[featuretype].keywords)
+            ])
+        )
 
         bbox = md.contents[featuretype].boundingBoxWGS84
         if bbox is not None:
@@ -633,7 +691,7 @@ def _parse_wfs(context, repos, record, identifier):
 
     # Derive a bbox based on aggregated featuretype bbox values
 
-    bbox_agg = util.bbox_from_polygons(bboxs)
+    bbox_agg = bbox_from_polygons(bboxs)
 
     if bbox_agg is not None:
         _set(context, serviceobj, 'pycsw:BoundingBox', bbox_agg)
@@ -660,7 +718,12 @@ def _parse_wcs(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/wcs')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(etree.tostring(md._capabilities)))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(etree.tostring(md._capabilities))
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -689,7 +752,7 @@ def _parse_wcs(context, repos, record, identifier):
 
     # generate record foreach coverage
 
-    LOGGER.debug('Harvesting %d WCS coverages ' % len(md.contents))
+    LOGGER.info('Harvesting %d WCS coverages ', len(md.contents))
 
     for coverage in md.contents:
         recobj = repos.dataset()
@@ -705,10 +768,16 @@ def _parse_wcs(context, repos, record, identifier):
         _set(context, recobj, 'pycsw:Abstract', md.contents[coverage].abstract)
         _set(context, recobj, 'pycsw:Keywords', ','.join(md.contents[coverage].keywords))
 
-        _set(context, recobj, 'pycsw:AnyText',
-             util.get_anytext([md.contents[coverage].title,
-                              md.contents[coverage].abstract,
-                              ','.join(md.contents[coverage].keywords)]))
+        _set(
+            context,
+            recobj,
+            'pycsw:AnyText',
+            util.get_anytext([
+                md.contents[coverage].title,
+                md.contents[coverage].abstract,
+                ','.join(md.contents[coverage].keywords)
+            ])
+        )
 
         bbox = md.contents[coverage].boundingBoxWGS84
         if bbox is not None:
@@ -730,7 +799,7 @@ def _parse_wcs(context, repos, record, identifier):
 
     # Derive a bbox based on aggregated coverage bbox values
 
-    bbox_agg = util.bbox_from_polygons(bboxs)
+    bbox_agg = bbox_from_polygons(bboxs)
 
     if bbox_agg is not None:
         _set(context, serviceobj, 'pycsw:BoundingBox', bbox_agg)
@@ -744,6 +813,7 @@ def _parse_wps(context, repos, record, identifier):
 
     from owslib.wps import WebProcessingService
 
+    recobjs = []
     serviceobj = repos.dataset()
 
     md = WebProcessingService(record)
@@ -754,7 +824,12 @@ def _parse_wps(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Schema', 'http://www.opengis.net/wps/1.0.0')
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(md._capabilities))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(md._capabilities)
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -782,7 +857,48 @@ def _parse_wps(context, repos, record, identifier):
     _set(context, serviceobj, 'pycsw:Links', '^'.join(links))
     _set(context, serviceobj, 'pycsw:XML', caps2iso(serviceobj, md, context))
 
-    return serviceobj
+    recobjs.append(serviceobj)
+
+    # generate record foreach process
+
+    LOGGER.info('Harvesting %d WPS processes', len(md.processes))
+
+    for process in md.processes:
+        recobj = repos.dataset()
+        identifier2 = '%s-%s' % (identifier, process.identifier)
+        _set(context, recobj, 'pycsw:Identifier', identifier2)
+        _set(context, recobj, 'pycsw:Typename', 'csw:Record')
+        _set(context, recobj, 'pycsw:Schema', 'http://www.opengis.net/wps/1.0.0')
+        _set(context, recobj, 'pycsw:MdSource', record)
+        _set(context, recobj, 'pycsw:InsertDate', util.get_today_and_now())
+        _set(context, recobj, 'pycsw:Type', 'software')
+        _set(context, recobj, 'pycsw:ParentIdentifier', identifier)
+        _set(context, recobj, 'pycsw:Title', process.title)
+        _set(context, recobj, 'pycsw:Abstract', process.abstract)
+
+        _set(
+            context,
+            recobj,
+            'pycsw:AnyText',
+            util.get_anytext([process.title, process.abstract])
+        )
+
+        params = {
+            'service': 'WPS',
+            'version': '1.0.0',
+            'request': 'DescribeProcess',
+            'identifier': process.identifier
+        }
+
+        links.append(
+        '%s,OGC-WPS DescribeProcess service (ver 1.0.0),OGC:WPS-1.0.0-http-describe-process,%s' % (identifier, build_get_url(md.url, {'service': 'WPS', 'version': '1.0.0', 'request': 'DescribeProcess', 'identifier': process.identifier})))
+
+        _set(context, recobj, 'pycsw:Links', '^'.join(links))
+        _set(context, recobj, 'pycsw:XML', caps2iso(recobj, md, context))
+
+        recobjs.append(recobj)
+
+    return recobjs
 
 
 def _parse_sos(context, repos, record, identifier, version):
@@ -806,7 +922,12 @@ def _parse_sos(context, repos, record, identifier, version):
     _set(context, serviceobj, 'pycsw:Schema', schema)
     _set(context, serviceobj, 'pycsw:MdSource', record)
     _set(context, serviceobj, 'pycsw:InsertDate', util.get_today_and_now())
-    _set(context, serviceobj, 'pycsw:AnyText', util.get_anytext(etree.tostring(md._capabilities)))
+    _set(
+        context,
+        serviceobj,
+        'pycsw:AnyText',
+        util.get_anytext(etree.tostring(md._capabilities))
+    )
     _set(context, serviceobj, 'pycsw:Type', 'service')
     _set(context, serviceobj, 'pycsw:Title', md.identification.title)
     _set(context, serviceobj, 'pycsw:Abstract', md.identification.abstract)
@@ -835,7 +956,7 @@ def _parse_sos(context, repos, record, identifier, version):
 
     # generate record foreach offering
 
-    LOGGER.debug('Harvesting %d SOS ObservationOffering\'s ', len(md.contents))
+    LOGGER.info('Harvesting %d SOS ObservationOffering\'s ', len(md.contents))
 
     for offering in md.contents:
         recobj = repos.dataset()
@@ -849,8 +970,12 @@ def _parse_sos(context, repos, record, identifier, version):
         _set(context, recobj, 'pycsw:ParentIdentifier', identifier)
         _set(context, recobj, 'pycsw:Title', md.contents[offering].description)
         _set(context, recobj, 'pycsw:Abstract', md.contents[offering].description)
-        _set(context, recobj, 'pycsw:TempExtent_begin', util.datetime2iso8601(md.contents[offering].begin_position))
-        _set(context, recobj, 'pycsw:TempExtent_end', util.datetime2iso8601(md.contents[offering].end_position))
+        begin = md.contents[offering].begin_position
+        end = md.contents[offering].end_position
+        _set(context, recobj, 'pycsw:TempExtent_begin',
+             util.datetime2iso8601(begin) if begin is not None else None)
+        _set(context, recobj, 'pycsw:TempExtent_end',
+             util.datetime2iso8601(end) if end is not None else None)
 
         #For observed_properties that have mmi url or urn, we simply want the observation name.
         observed_properties = []
@@ -884,7 +1009,7 @@ def _parse_sos(context, repos, record, identifier, version):
 
     # Derive a bbox based on aggregated featuretype bbox values
 
-    bbox_agg = util.bbox_from_polygons(bboxs)
+    bbox_agg = bbox_from_polygons(bboxs)
 
     if bbox_agg is not None:
         _set(context, serviceobj, 'pycsw:BoundingBox', bbox_agg)
@@ -1089,11 +1214,15 @@ def _parse_gm03(context, repos, exml):
 def _parse_iso(context, repos, exml):
 
     from owslib.iso import MD_Metadata
+    from owslib.iso_che import CHE_MD_Metadata
 
     recobj = repos.dataset()
     links = []
 
-    md = MD_Metadata(exml)
+    if exml.tag == '{http://www.geocat.ch/2008/che}CHE_MD_Metadata':
+        md = CHE_MD_Metadata(exml)
+    else:
+        md = MD_Metadata(exml)
 
     _set(context, recobj, 'pycsw:Identifier', md.identifier)
     _set(context, recobj, 'pycsw:Typename', 'gmd:MD_Metadata')
@@ -1256,7 +1385,7 @@ def _parse_iso(context, repos, exml):
                         (scpt.name, scpt.description, scpt.protocol, scpt.url)
                         links.append(linkstr)
     except Exception as err:  # srv: identification does not exist
-        LOGGER.debug('no srv:SV_ServiceIdentification links found')
+        LOGGER.exception('no srv:SV_ServiceIdentification links found')
 
     if len(links) > 0:
         _set(context, recobj, 'pycsw:Links', '^'.join(links))
@@ -1354,3 +1483,29 @@ def caps2iso(record, caps, context):
     queryables = dict(apiso_obj.repository['queryables']['SupportedISOQueryables'].items())
     iso_xml = apiso_obj.write_record(record, 'full', 'http://www.isotc211.org/2005/gmd', queryables, caps)
     return etree.tostring(iso_xml)
+
+
+def bbox_from_polygons(bboxs):
+    """Derive an aggregated bbox from n polygons
+
+    Parameters
+    ----------
+    bboxs: list
+        A sequence of strings containing Well-Known Text representations of
+        polygons
+
+    Returns
+    -------
+    str
+        Well-Known Text representation of the aggregated bounding box for
+        all the input polygons
+    """
+
+    try:
+        multi_pol = MultiPolygon(
+            [loads(bbox) for bbox in bboxs]
+        )
+        bstr = ",".join(["{0:.2f}".format(b) for b in multi_pol.bounds])
+        return util.bbox2wktpolygon(bstr)
+    except Exception as err:
+        raise RuntimeError('Cannot aggregate polygons: %s' % str(err))
