@@ -42,6 +42,7 @@ from pycsw.ogc.csw.cql import cql2fes1
 from pycsw.plugins.profiles import profile as pprofile
 import pycsw.plugins.outputschemas
 from pycsw.core import config, log, metadata, util
+from pycsw.core.formats.fmt_json import xml2dict
 from pycsw.ogc.fes import fes1
 import logging
 
@@ -112,7 +113,7 @@ class Csw2(object):
         metadata_main = dict(self.parent.config.items('metadata:main'))
 
         if serviceidentification:
-            LOGGER.debug('Writing section ServiceIdentification.')
+            LOGGER.info('Writing section ServiceIdentification')
 
             serviceidentification = etree.SubElement(node, \
             util.nspath_eval('ows:ServiceIdentification',
@@ -159,7 +160,7 @@ class Csw2(object):
             metadata_main.get('identification_accessconstraints', 'missing')
 
         if serviceprovider:
-            LOGGER.debug('Writing section ServiceProvider.')
+            LOGGER.info('Writing section ServiceProvider')
             serviceprovider = etree.SubElement(node,
             util.nspath_eval('ows:ServiceProvider', self.parent.context.namespaces))
 
@@ -261,7 +262,7 @@ class Csw2(object):
             metadata_main.get('contact_role', 'missing')
 
         if operationsmetadata:
-            LOGGER.debug('Writing section OperationsMetadata.')
+            LOGGER.info('Writing section OperationsMetadata')
             operationsmetadata = etree.SubElement(node,
             util.nspath_eval('ows:OperationsMetadata',
             self.parent.context.namespaces))
@@ -359,7 +360,7 @@ class Csw2(object):
                         operationsmetadata.append(ecnode)
 
         # always write out Filter_Capabilities
-        LOGGER.debug('Writing section Filter_Capabilities.')
+        LOGGER.info('Writing section Filter_Capabilities')
         fltcaps = etree.SubElement(node,
         util.nspath_eval('ogc:Filter_Capabilities', self.parent.context.namespaces))
 
@@ -467,7 +468,7 @@ class Csw2(object):
                 return self.exceptionreport('InvalidParameterValue',
                 'typename', 'Typename not qualified: %s' % typename)
             if typename == 'csw:Record':   # load core schema
-                LOGGER.debug('Writing csw:Record schema.')
+                LOGGER.info('Writing csw:Record schema')
                 schemacomponent = etree.SubElement(node,
                 util.nspath_eval('csw:SchemaComponent', self.parent.context.namespaces),
                 schemaLanguage='XMLSCHEMA',
@@ -508,7 +509,7 @@ class Csw2(object):
 
         if 'parametername' in self.parent.kvp:
             for pname in self.parent.kvp['parametername'].split(','):
-                LOGGER.debug('Parsing parametername %s.' % pname)
+                LOGGER.info('Parsing parametername %s', pname)
                 domainvalue = etree.SubElement(node,
                 util.nspath_eval('csw:DomainValues', self.parent.context.namespaces),
                 type='csw:Record')
@@ -533,7 +534,7 @@ class Csw2(object):
 
         if 'propertyname' in self.parent.kvp:
             for pname in self.parent.kvp['propertyname'].split(','):
-                LOGGER.debug('Parsing propertyname %s.' % pname)
+                LOGGER.info('Parsing propertyname %s', pname)
 
                 if pname.find('/') == 0:  # it's an XPath
                     pname2 = pname
@@ -562,10 +563,9 @@ class Csw2(object):
                 self.parent.context.namespaces)).text = pname
 
                 try:
-                    LOGGER.debug(
+                    LOGGER.info(
                     'Querying repository property %s, typename %s, \
-                    domainquerytype %s.' % \
-                    (pname2, dvtype, self.parent.domainquerytype))
+                    domainquerytype %s',  pname2, dvtype, self.parent.domainquerytype)
 
                     count = False
 
@@ -576,7 +576,7 @@ class Csw2(object):
                     results = self.parent.repository.query_domain(
                     pname2, dvtype, self.parent.domainquerytype, count)
 
-                    LOGGER.debug('Results: %s' % str(len(results)))
+                    LOGGER.debug('Results: %d', len(results))
 
                     if self.parent.domainquerytype == 'range':
                         rangeofvalues = etree.SubElement(domainvalue,
@@ -606,8 +606,9 @@ class Csw2(object):
                                 util.nspath_eval('csw:Value',
                                 self.parent.context.namespaces)).text = val
                 except Exception as err:
-                    LOGGER.debug('No results for propertyname %s: %s.' %
-                    (pname2, str(err)))
+                    # here we fail silently back to the client because
+                    # CSW tells us to
+                    LOGGER.exception('No results for propertynames')
         return node
 
     def getrecords(self):
@@ -715,7 +716,7 @@ class Csw2(object):
             tmp_filter = opensearch.kvp2filterxml(self.parent.kvp, self.parent.context)
             if tmp_filter is not "":
                 self.parent.kvp['constraint'] = tmp_filter
-                LOGGER.debug('OpenSearch Geo/Time parameters to Filter: %s.' % self.parent.kvp['constraint'])
+                LOGGER.debug('OpenSearch Geo/Time parameters to Filter: %s.', self.parent.kvp['constraint'])
 
         if self.parent.requesttype == 'GET':
             if 'constraint' in self.parent.kvp:
@@ -734,7 +735,7 @@ class Csw2(object):
                 if self.parent.kvp['constraintlanguage'] == 'CQL_TEXT':
                     tmp = self.parent.kvp['constraint']
                     try:
-                        LOGGER.debug('Transforming CQL into fes1')
+                        LOGGER.info('Transforming CQL into fes1')
                         LOGGER.debug('CQL: %s', tmp)
                         self.parent.kvp['constraint'] = {}
                         self.parent.kvp['constraint']['type'] = 'filter'
@@ -742,9 +743,9 @@ class Csw2(object):
                         self.parent.kvp['constraint']['where'], self.parent.kvp['constraint']['values'] = fes1.parse(cql,
                         self.parent.repository.queryables['_all'], self.parent.repository.dbtype,
                         self.parent.context.namespaces, self.parent.orm, self.parent.language['text'], self.parent.repository.fts)
+                        self.parent.kvp['constraint']['_dict'] = xml2dict(etree.tostring(cql), self.parent.context.namespaces)
                     except Exception as err:
-                        LOGGER.error('Invalid CQL query %s', tmp)
-                        LOGGER.error('Error message: %s', err, exc_info=True)
+                        LOGGER.exception('Invalid CQL query %s', tmp)
                         return self.exceptionreport('InvalidParameterValue',
                         'constraint', 'Invalid Filter syntax')
                 elif self.parent.kvp['constraintlanguage'] == 'FILTER':
@@ -752,12 +753,11 @@ class Csw2(object):
                     try:
                         schema = os.path.join(self.parent.config.get('server', 'home'),
                         'core', 'schemas', 'ogc', 'filter', '1.1.0', 'filter.xsd')
-                        LOGGER.debug('Validating Filter %s.' %
-                        self.parent.kvp['constraint'])
+                        LOGGER.info('Validating Filter %s', self.parent.kvp['constraint'])
                         schema = etree.XMLSchema(file=schema)
                         parser = etree.XMLParser(schema=schema, resolve_entities=False)
                         doc = etree.fromstring(self.parent.kvp['constraint'], parser)
-                        LOGGER.debug('Filter is valid XML.')
+                        LOGGER.debug('Filter is valid XML')
                         self.parent.kvp['constraint'] = {}
                         self.parent.kvp['constraint']['type'] = 'filter'
                         self.parent.kvp['constraint']['where'], self.parent.kvp['constraint']['values'] = \
@@ -765,11 +765,11 @@ class Csw2(object):
                         self.parent.repository.queryables['_all'],
                         self.parent.repository.dbtype,
                         self.parent.context.namespaces, self.parent.orm, self.parent.language['text'], self.parent.repository.fts)
+                        self.parent.kvp['constraint']['_dict'] = xml2dict(etree.tostring(doc), self.parent.context.namespaces)
                     except Exception as err:
                         errortext = \
                         'Exception: document not valid.\nError: %s.' % str(err)
-
-                        LOGGER.debug(errortext)
+                        LOGGER.exception(errortext)
                         return self.exceptionreport('InvalidParameterValue',
                         'constraint', 'Invalid Filter query: %s' % errortext)
             else:
@@ -778,7 +778,7 @@ class Csw2(object):
         if 'sortby' not in self.parent.kvp:
             self.parent.kvp['sortby'] = None
         elif 'sortby' in self.parent.kvp and self.parent.requesttype == 'GET':
-            LOGGER.debug('Sorted query specified.')
+            LOGGER.debug('Sorted query specified')
             tmp = self.parent.kvp['sortby']
             self.parent.kvp['sortby'] = {}
 
@@ -813,9 +813,9 @@ class Csw2(object):
 
         # query repository
         LOGGER.debug('Querying repository with constraint: %s,\
-        sortby: %s, typenames: %s, maxrecords: %s, startposition: %s.' %
-        (self.parent.kvp['constraint'], self.parent.kvp['sortby'], self.parent.kvp['typenames'],
-        self.parent.kvp['maxrecords'], self.parent.kvp['startposition']))
+        sortby: %s, typenames: %s, maxrecords: %s, startposition: %s',
+        self.parent.kvp['constraint'], self.parent.kvp['sortby'], self.parent.kvp['typenames'],
+        self.parent.kvp['maxrecords'], self.parent.kvp['startposition'])
 
         try:
             matched, results = self.parent.repository.query(
@@ -824,8 +824,8 @@ class Csw2(object):
             maxrecords=self.parent.kvp['maxrecords'],
             startposition=int(self.parent.kvp['startposition'])-1)
         except Exception as err:
-            LOGGER.debug('Invalid query syntax.  Query: %s', self.parent.kvp['constraint'])
-            LOGGER.debug('Invalid query syntax.  Result: %s', err)
+            LOGGER.exception('Invalid query syntax.  Query: %s', self.parent.kvp['constraint'])
+            LOGGER.exception('Invalid query syntax.  Result: %s', err)
             return self.exceptionreport('InvalidParameterValue', 'constraint',
             'Invalid query syntax')
 
@@ -836,7 +836,7 @@ class Csw2(object):
             self.parent.kvp['distributedsearch'] and self.parent.kvp['hopcount'] > 0):
             # do distributed search
 
-            LOGGER.debug('DistributedSearch specified (hopCount: %s).' %
+            LOGGER.debug('DistributedSearch specified (hopCount: %s).',
             self.parent.kvp['hopcount'])
 
             from owslib.csw import CatalogueServiceWeb
@@ -844,7 +844,7 @@ class Csw2(object):
             for fedcat in \
             self.parent.config.get('server', 'federatedcatalogues').split(','):
                 LOGGER.debug('Performing distributed search on federated \
-                catalogue: %s.' % fedcat)
+                catalogue: %s.', fedcat)
                 remotecsw = CatalogueServiceWeb(fedcat, skip_caps=True)
                 try:
                     remotecsw.getrecords2(xml=self.parent.request,
@@ -853,7 +853,7 @@ class Csw2(object):
                     if hasattr(remotecsw, 'results'):
                         LOGGER.debug(
                         'Distributed search results from catalogue \
-                        %s: %s.' % (fedcat, remotecsw.results))
+                        %s: %s.', fedcat, remotecsw.results)
 
                         remotecsw_matches = int(remotecsw.results['matches'])
                         plural = 's' if remotecsw_matches != 1 else ''
@@ -868,12 +868,12 @@ class Csw2(object):
                     error_string = 'remote CSW %s returned exception: ' % fedcat
                     dsresults.append(etree.Comment(
                     ' %s\n\n%s ' % (error_string, err)))
-                    LOGGER.debug(str(err))
+                    LOGGER.exception(error_string)
                 except Exception as err:
                     error_string = 'remote CSW %s returned error: ' % fedcat
                     dsresults.append(etree.Comment(
                     ' %s\n\n%s ' % (error_string, err)))
-                    LOGGER.debug(str(err))
+                    LOGGER.exception(error_string)
 
         if int(matched) == 0:
             returned = nextrecord = '0'
@@ -889,8 +889,8 @@ class Csw2(object):
                     nextrecord = str(int(self.parent.kvp['startposition']) + \
                     int(self.parent.kvp['maxrecords']))
 
-        LOGGER.debug('Results: matched: %s, returned: %s, next: %s.' % \
-        (matched, returned, nextrecord))
+        LOGGER.debug('Results: matched: %s, returned: %s, next: %s',
+        matched, returned, nextrecord)
 
         node = etree.Element(util.nspath_eval('csw:GetRecordsResponse',
         self.parent.context.namespaces),
@@ -922,7 +922,7 @@ class Csw2(object):
 
         if 'where' not in self.parent.kvp['constraint'] \
         and self.parent.kvp['resulttype'] is None:
-            LOGGER.debug('Empty result set returned.')
+            LOGGER.debug('Empty result set returned')
             return node
 
         if self.parent.kvp['resulttype'] == 'hits':
@@ -934,8 +934,8 @@ class Csw2(object):
                 max1 = len(results)
             else:
                 max1 = int(self.parent.kvp['startposition']) + (int(self.parent.kvp['maxrecords'])-1)
-            LOGGER.debug('Presenting records %s - %s.' %
-            (self.parent.kvp['startposition'], max1))
+            LOGGER.info('Presenting records %s - %s',
+            self.parent.kvp['startposition'], max1)
 
             for res in results:
                 try:
@@ -957,9 +957,11 @@ class Csw2(object):
                                 typename = self.parent.profiles['loaded'][prof].typename
                                 break
 
-                        util.transform_mappings(self.parent.repository.queryables['_all'],
-                        self.parent.context.model['typenames'][typename]\
-                        ['mappings']['csw:Record'], reverse=True)
+                        util.transform_mappings(
+                            self.parent.repository.queryables['_all'],
+                            self.parent.context.model['typenames'][typename][
+                                'mappings']['csw:Record']
+                        )
 
                         searchresults.append(self._write_record(
                         res, self.parent.repository.queryables['_all']))
@@ -1038,11 +1040,11 @@ class Csw2(object):
         (self.parent.context.namespaces['csw'], self.parent.config.get('server', 'ogc_schemas_base'))
 
         # query repository
-        LOGGER.debug('Querying repository with ids: %s.' % self.parent.kvp['id'][0])
+        LOGGER.info('Querying repository with ids: %s', self.parent.kvp['id'][0])
         results = self.parent.repository.query_ids(self.parent.kvp['id'])
 
         if raw:  # GetRepositoryItem request
-            LOGGER.debug('GetRepositoryItem request.')
+            LOGGER.debug('GetRepositoryItem request')
             if len(results) > 0:
                 return etree.fromstring(util.getqattr(results[0],
                 self.parent.context.md_core_model['mappings']['pycsw:XML']), self.parent.context.parser)
@@ -1067,9 +1069,11 @@ class Csw2(object):
                         break
 
                 if typename is not None:
-                    util.transform_mappings(self.parent.repository.queryables['_all'],
-                    self.parent.context.model['typenames'][typename]\
-                    ['mappings']['csw:Record'], reverse=True)
+                    util.transform_mappings(
+                        self.parent.repository.queryables['_all'],
+                        self.parent.context.model['typenames'][typename][
+                            'mappings']['csw:Record']
+                    )
 
                 node.append(self._write_record(
                 result, self.parent.repository.queryables['_all']))
@@ -1112,7 +1116,7 @@ class Csw2(object):
 
         insertresults = []
 
-        LOGGER.debug('Transaction list: %s' % self.parent.kvp['transactions'])
+        LOGGER.debug('Transaction list: %s', self.parent.kvp['transactions'])
 
         for ttype in self.parent.kvp['transactions']:
             if ttype['type'] == 'insert':
@@ -1120,11 +1124,12 @@ class Csw2(object):
                     record = metadata.parse_record(self.parent.context,
                     ttype['xml'], self.parent.repository)[0]
                 except Exception as err:
+                    LOGGER.exception('Transaction (insert) failed')
                     return self.exceptionreport('NoApplicableCode', 'insert',
                     'Transaction (insert) failed: record parsing failed: %s' \
                     % str(err))
 
-                LOGGER.debug('Transaction operation: %s' % record)
+                LOGGER.debug('Transaction operation: %s', record)
 
                 if not hasattr(record,
                 self.parent.context.md_core_model['mappings']['pycsw:Identifier']):
@@ -1160,14 +1165,12 @@ class Csw2(object):
                         % str(err))
 
                     # query repository to see if record already exists
-                    LOGGER.debug('checking if record exists (%s)' % \
-                    identifier)
+                    LOGGER.info('checking if record exists (%s)', identifier)
 
                     results = self.parent.repository.query_ids(ids=[identifier])
 
                     if len(results) == 0:
-                        LOGGER.debug('id %s does not exist in repository' % \
-                        identifier)
+                        LOGGER.debug('id %s does not exist in repository', identifier)
                     else:  # existing record, it's an update
                         try:
                             self.parent.repository.update(record)
@@ -1196,13 +1199,13 @@ class Csw2(object):
                             rp['rp']= \
                             self.parent.repository.queryables['_all'][rp['name']]
 
-                    LOGGER.debug('Record Properties: %s.' %
-                    ttype['recordproperty'])
+                    LOGGER.debug('Record Properties: %s.', ttype['recordproperty'])
                     try:
                         updated += self.parent.repository.update(record=None,
                         recprops=ttype['recordproperty'],
                         constraint=ttype['constraint'])
                     except Exception as err:
+                        LOGGER.exception('Transaction (updated) failed')
                         return self.exceptionreport('NoApplicableCode',
                         'update',
                         'Transaction (update) failed: %s.' % str(err))
@@ -1260,26 +1263,26 @@ class Csw2(object):
         if (self.parent.kvp['resourcetype'].find('opengis.net') == -1 and
             self.parent.kvp['resourcetype'].find('urn:geoss:waf') == -1):
             # fetch content-based resource
-            LOGGER.debug('Fetching resource %s' % self.parent.kvp['source'])
+            LOGGER.debug('Fetching resource %s', self.parent.kvp['source'])
             try:
                 content = util.http_request('GET', self.parent.kvp['source'])
             except Exception as err:
                 errortext = 'Error fetching resource %s.\nError: %s.' % \
                 (self.parent.kvp['source'], str(err))
-                LOGGER.debug(errortext)
+                LOGGER.exception(errortext)
                 return self.exceptionreport('InvalidParameterValue', 'source',
                 errortext)
         else:  # it's a service URL
             content = self.parent.kvp['source']
             # query repository to see if service already exists
-            LOGGER.debug('checking if service exists (%s)' % content)
+            LOGGER.info('checking if service exists (%s)', content)
             results = self.parent.repository.query_source(content)
 
             if len(results) > 0:  # exists, keep identifier for update
                 LOGGER.debug('Service already exists, keeping identifier and results')
                 service_identifier = getattr(results[0], self.parent.context.md_core_model['mappings']['pycsw:Identifier'])
                 service_results = results
-                LOGGER.debug('Identifier is %s' % service_identifier)
+                LOGGER.debug('Identifier is %s', service_identifier)
             #    return self.exceptionreport('NoApplicableCode', 'source',
             #    'Insert failed: service %s already in repository' % content)
 
@@ -1291,6 +1294,7 @@ class Csw2(object):
                 ir = self.parent.repository.insert(self.parent.kvp['resourcetype'], self.parent.kvp['source'])
                 inserted = len(ir)
             except Exception as err:
+                LOGGER.exception('Harvest (insert) failed')
                 return self.exceptionreport('NoApplicableCode',
                 'source', 'Harvest (insert) failed: %s.' % str(err))
         else:
@@ -1308,7 +1312,7 @@ class Csw2(object):
             updated = 0
             ir = []
 
-            LOGGER.debug('Total Records parsed: %d' % len(records_parsed))
+            LOGGER.debug('Total Records parsed: %d', len(records_parsed))
             for record in records_parsed:
                 if self.parent.kvp['resourcetype'] == 'urn:geoss:waf':
                     src = record.source
@@ -1335,14 +1339,14 @@ class Csw2(object):
                 record_identifier = getattr(record, self.parent.context.md_core_model['mappings']['pycsw:Identifier'])
 
                 if record_type == 'service' and service_identifier is not None:  # service endpoint
-                    LOGGER.debug('Replacing service identifier from %s to %s' % (record_identifier, service_identifier))
+                    LOGGER.info('Replacing service identifier from %s to %s', record_identifier, service_identifier)
                     old_identifier = record_identifier
                     identifier = record_identifier = service_identifier
                 if (record_type != 'service' and service_identifier is not None
                     and old_identifier is not None):  # service resource
                     if record_identifier.find(old_identifier) != -1:
                         new_identifier = record_identifier.replace(old_identifier, service_identifier)
-                        LOGGER.debug('Replacing service resource identifier from %s to %s' % (record_identifier, new_identifier))
+                        LOGGER.debug('Replacing service resource identifier from %s to %s', record_identifier, new_identifier)
                         identifier = record_identifier = new_identifier
 
                 ir.append({'identifier': identifier, 'title': title})
@@ -1350,11 +1354,11 @@ class Csw2(object):
                 results = []
                 if not self.parent.config.has_option('repository', 'source'):
                     # query repository to see if record already exists
-                    LOGGER.debug('checking if record exists (%s)' % identifier)
+                    LOGGER.info('checking if record exists (%s)', identifier)
                     results = self.parent.repository.query_ids(ids=[identifier])
 
                     if len(results) == 0:  # check for service identifier
-                        LOGGER.debug('checking if service id exists (%s)' % service_identifier)
+                        LOGGER.info('checking if service id exists (%s)', service_identifier)
                         results = self.parent.repository.query_ids(ids=[service_identifier])
 
                 LOGGER.debug(str(results))
@@ -1543,12 +1547,13 @@ class Csw2(object):
 
         tmp = element.find(util.nspath_eval('ogc:Filter', self.parent.context.namespaces))
         if tmp is not None:
-            LOGGER.debug('Filter constraint specified.')
+            LOGGER.debug('Filter constraint specified')
             try:
                 query['type'] = 'filter'
                 query['where'], query['values'] = fes1.parse(tmp,
                 self.parent.repository.queryables['_all'], self.parent.repository.dbtype,
                 self.parent.context.namespaces, self.parent.orm, self.parent.language['text'], self.parent.repository.fts)
+                query['_dict'] = xml2dict(etree.tostring(tmp), self.parent.context.namespaces)
             except Exception as err:
                 return 'Invalid Filter request: %s' % err
 
@@ -1556,15 +1561,16 @@ class Csw2(object):
         if tmp is not None:
             LOGGER.debug('CQL specified: %s.', tmp.text)
             try:
-                LOGGER.debug('Transforming CQL into OGC Filter')
+                LOGGER.info('Transforming CQL into OGC Filter')
                 query['type'] = 'filter'
                 cql = cql2fes1(tmp.text, self.parent.context.namespaces)
                 query['where'], query['values'] = fes1.parse(cql,
                 self.parent.repository.queryables['_all'], self.parent.repository.dbtype,
                 self.parent.context.namespaces, self.parent.orm, self.parent.language['text'], self.parent.repository.fts)
+                query['_dict'] = xml2dict(etree.tostring(cql), self.parent.context.namespaces)
             except Exception as err:
-                LOGGER.error('Invalid CQL request: %s', tmp.text)
-                LOGGER.error('Error message: %s', err, exc_info=True)
+                LOGGER.exception('Invalid CQL request: %s', tmp.text)
+                LOGGER.exception('Error message: %s', err)
                 return 'Invalid CQL request'
         return query
 
@@ -1573,20 +1579,19 @@ class Csw2(object):
 
         request = {}
         try:
-            LOGGER.debug('Parsing %s.' % postdata)
+            LOGGER.info('Parsing %s', postdata)
             doc = etree.fromstring(postdata, self.parent.context.parser)
         except Exception as err:
             errortext = \
             'Exception: document not well-formed.\nError: %s.' % str(err)
-
-            LOGGER.debug(errortext)
+            LOGGER.exception(errortext)
             return errortext
 
         # if this is a SOAP request, get to SOAP-ENV:Body/csw:*
         if (doc.tag == util.nspath_eval('soapenv:Envelope',
             self.parent.context.namespaces)):
 
-            LOGGER.debug('SOAP request specified.')
+            LOGGER.debug('SOAP request specified')
             self.parent.soap = True
 
             doc = doc.find(
@@ -1612,7 +1617,7 @@ class Csw2(object):
             len(doc.xpath('//csw:Update/child::*',
             namespaces=self.parent.context.namespaces)) == 0:
 
-                LOGGER.debug('Validating %s.' % postdata)
+                LOGGER.info('Validating %s', postdata)
                 schema = etree.XMLSchema(file=schema)
                 parser = etree.XMLParser(schema=schema, resolve_entities=False)
                 if hasattr(self.parent, 'soap') and self.parent.soap:
@@ -1626,11 +1631,11 @@ class Csw2(object):
         except Exception as err:
             errortext = \
             'Exception: the document is not valid.\nError: %s' % str(err)
-            LOGGER.debug(errortext)
+            LOGGER.exception(errortext)
             return errortext
 
-        request['request'] = util.xmltag_split(doc.tag)
-        LOGGER.debug('Request operation %s specified.' % request['request'])
+        request['request'] = etree.QName(doc).localname
+        LOGGER.debug('Request operation %s specified.', request['request'])
         tmp = doc.find('.').attrib.get('service')
         if tmp is not None:
             request['service'] = tmp
@@ -1745,12 +1750,12 @@ class Csw2(object):
                     return 'Invalid Constraint: %s' % request['constraint']
             else:
                 LOGGER.debug('No csw:Constraint (ogc:Filter or csw:CqlText) \
-                specified.')
+                specified')
 
             tmp = doc.find(util.nspath_eval('csw:Query/ogc:SortBy',
                   self.parent.context.namespaces))
             if tmp is not None:
-                LOGGER.debug('Sorted query specified.')
+                LOGGER.debug('Sorted query specified')
                 request['sortby'] = {}
 
 
@@ -1769,7 +1774,7 @@ class Csw2(object):
                 except Exception as err:
                     errortext = \
                     'Invalid ogc:SortProperty/ogc:PropertyName: %s' % str(err)
-                    LOGGER.debug(errortext)
+                    LOGGER.exception(errortext)
                     return errortext
 
                 tmp2 =  tmp.find(util.nspath_eval(
