@@ -31,7 +31,9 @@
 # =================================================================
 
 import logging
+
 from pycsw.core import util
+from pycsw.core.etree import etree
 from pycsw.ogc.gml import gml3
 
 LOGGER = logging.getLogger(__name__)
@@ -79,8 +81,9 @@ def parse(element, queryables, dbtype, nsmap, orm='sqlalchemy', language='englis
 
     tmp = element.xpath('ogc:And|ogc:Or|ogc:Not', namespaces=nsmap)
     if len(tmp) > 0:  # this is binary logic query
-        boq = ' %s ' % util.xmltag_split(tmp[0].tag).lower()
-        LOGGER.debug('Binary logic detected; operator=%s' % boq)
+        element_name = etree.QName(tmp[0]).localname
+        boq = ' %s ' % element_name.lower()
+        LOGGER.debug('Binary logic detected; operator=%s', boq)
         tmp = tmp[0]
     else:
         tmp = element
@@ -146,7 +149,7 @@ def parse(element, queryables, dbtype, nsmap, orm='sqlalchemy', language='englis
                 pval = elem.find(util.nspath_eval('ogc:Literal', nsmap)).text
 
         com_op = _get_comparison_operator(elem)
-        LOGGER.debug('Comparison operator: %s' % com_op)
+        LOGGER.debug('Comparison operator: %s', com_op)
 
         # if this is a case insensitive search
         # then set the DB-specific LIKE comparison operator
@@ -235,20 +238,20 @@ def parse(element, queryables, dbtype, nsmap, orm='sqlalchemy', language='englis
             if child_not.tag in \
                 [util.nspath_eval('ogc:%s' % n, nsmap) for n in
                     MODEL['SpatialOperators']['values']]:
-                LOGGER.debug('ogc:Not / spatial operator detected: %s' % child.tag)
+                LOGGER.debug('ogc:Not / spatial operator detected: %s', child.tag)
                 queries.append("%s = %s" %
                                (_get_spatial_operator(
                                    queryables['pycsw:BoundingBox'],
                                    child.xpath('child::*')[0], dbtype, nsmap),
                                    boolean_false))
             else:
-                LOGGER.debug('ogc:Not / comparison operator detected: %s' % child.tag)
+                LOGGER.debug('ogc:Not / comparison operator detected: %s', child.tag)
                 queries.append('not %s' % _get_comparison_expression(child_not))
 
         elif child.tag in \
             [util.nspath_eval('ogc:%s' % n, nsmap) for n in
                 MODEL['SpatialOperators']['values']]:
-            LOGGER.debug('spatial operator detected: %s' % child.tag)
+            LOGGER.debug('spatial operator detected: %s', child.tag)
             if boq is not None and boq == ' not ':
                 # for ogc:Not spatial queries in PostGIS we must explictly
                 # test that pycsw:BoundingBox is null as well
@@ -277,9 +280,10 @@ def parse(element, queryables, dbtype, nsmap, orm='sqlalchemy', language='englis
             values.append(child.attrib.get('fid'))
         else:  # comparison operator
             LOGGER.debug('Comparison operator processing')
-            tagname = ' %s ' % util.xmltag_split(child.tag).lower()
+            child_tag_name = etree.QName(child).localname
+            tagname = ' %s ' % child_tag_name.lower()
             if tagname in [' or ', ' and ']:  # this is a nested binary logic query
-                LOGGER.debug('Nested binary logic detected; operator=%s' % tagname)
+                LOGGER.debug('Nested binary logic detected; operator=%s', tagname)
                 for child2 in child.xpath('child::*'):
                     queries_nested.append(_get_comparison_expression(child2))
                 queries.append('(%s)' % tagname.join(queries_nested))
@@ -313,9 +317,9 @@ def _get_spatial_operator(geomattr, element, dbtype, nsmap, postgis_geometry_col
     #make decision to apply spatial ranking to results
     set_spatial_ranking(geometry)
 
-    spatial_predicate = util.xmltag_split(element.tag).lower()
+    spatial_predicate = etree.QName(element).localname.lower()
 
-    LOGGER.debug('Spatial predicate: %s' % spatial_predicate)
+    LOGGER.debug('Spatial predicate: %s', spatial_predicate)
 
     if dbtype == 'mysql':  # adjust spatial query for MySQL
         LOGGER.debug('Adjusting spatial query for MySQL')
@@ -382,7 +386,8 @@ def _get_spatial_operator(geomattr, element, dbtype, nsmap, postgis_geometry_col
 def _get_comparison_operator(element):
     """return the SQL operator based on Filter query"""
 
-    return MODEL['ComparisonOperators']['ogc:%s' % util.xmltag_split(element.tag)]['opvalue']
+    element_name = etree.QName(element).localname
+    return MODEL['ComparisonOperators']['ogc:%s' % element_name]['opvalue']
 
 def set_spatial_ranking(geometry):
     """Given that we have a spatial query in ogc:Filter we check the type of geometry
